/*------------------------------------------------------------------------
Module:        example.c
Author:        Ludd
Project:       GTK example code
State:
Creation Date: Mar 17, 2000
Description:   This is an example program for Wedit IDE.
If you succeed in compiling and running it, your Wedit and GTK installation
is set up correctly.
------------------------------------------------------------------------*/
#include <gtk/gtk.h>
#include <stdlib.h>
#include <time.h>

#define NONE	0
#define CROSS	1
#define	CIRCLE	2

int sheet[3][3];
GtkWidget *drawingarea1, *statusbar1;
gboolean win;

void draw_blank(GtkWidget *widget, int x, int y, int w, int h)
{
	gdk_draw_rectangle (widget->window,
		widget->style->bg_gc[0],
		TRUE,
		x + 1,
		y + 2,
		w - 2,
		h - 2);
}
void draw_circle(GtkWidget *widget, int x, int y, int w, int h)
{
	gdk_draw_arc(widget->window,
		widget->style->mid_gc[0],
		FALSE,
		x + 3,
		y + 3,
		w - 6,
		h - 6,
		0,
		23040);
}

void draw_cross(GtkWidget *widget, int x, int y, int w, int h)
{
	gdk_draw_line (widget->window,
		widget->style->mid_gc[1],
		x + 3,
		y + 3,
		x + w - 6,
		y + h - 6);
	gdk_draw_line (widget->window,
		widget->style->mid_gc[1],
		x + w - 6,
		y + 3,
		x + 3,
		y + h - 6);
}

void show_message(gchar *message)
{
	gtk_statusbar_push(GTK_STATUSBAR(statusbar1), 1, message);
}

void reset_game()
{
	int x,y;
	GdkColormap *colormap;
	GdkColor color;

	show_message("Tic Tac Toe");

	/* clear deck */
	for (x = 0; x < 3; x++)
		for (y = 0; y < 3; y++)
			sheet[x][y] = NONE;

	/* prepare blue and red colors in the drawingarea1 */
	colormap = gtk_widget_get_colormap (GTK_WIDGET (drawingarea1));
	color.red = 0;
	color.green = 0;
	color.blue = 65535;
	color.pixel = 0;
	gdk_colormap_alloc_color (colormap, &color, FALSE, FALSE);

	if (drawingarea1->style->mid_gc[0])
		gdk_gc_set_foreground(drawingarea1->style->mid_gc[0], &color);

	color.red = 65535;
	color.green = 0;
	color.blue = 0;
	color.pixel = 0;
	gdk_colormap_alloc_color (colormap, &color, FALSE, FALSE);

	if (drawingarea1->style->mid_gc[1])
		gdk_gc_set_foreground(drawingarea1->style->mid_gc[1], &color);

	/* redraw deck */
	gtk_widget_queue_draw(drawingarea1);

	win = FALSE;
}

/*------------------------------------------------------------------------
Procedure:     check_win
Purpose:       Check if specified player won
Input:         Player color (CROSS or CIRCLE)
Output:        TRUE if won, FALSE otherwise
Errors:
------------------------------------------------------------------------*/
gboolean check_win(int player)
{
	int x, y, count;

	/* check horizontally */
	for (y = 0; y < 3; y++) {
		count = 0;
		for (x = 0; x < 3; x++)
			if (sheet[x][y] == player)
				count++;
		if (count == 3)
			return TRUE;
	}

	/* check vertically */
	for (x = 0; x < 3; x++) {
		count = 0;
		for (y = 0; y < 3; y++)
			if (sheet[x][y] == player)
				count++;
		if (count == 3)
			return TRUE;
	}

	/* check diagonally */
	count = 0;
	for (x = 0; x < 3; x++)
		if (sheet[x][x] == player)
			count++;
	if (count == 3)
		return TRUE;

	count = 0;
	for (x = 0; x < 3; x++)
		if (sheet[2-x][x] == player)
			count++;
	if (count == 3)
		return TRUE;

	/* no win yet */
	return FALSE;

}

/*------------------------------------------------------------------------
Procedure:     check_draw
Purpose:       Check if all fields are full
Input:
Output:        TRUE if deck is full
Errors:
------------------------------------------------------------------------*/
gboolean check_draw()
{
	int x, y, count = 0;

	for (x = 0; x < 3; x++)
		for (y = 0; y < 3; y++)
			if (sheet[x][y] != NONE)
				count++;

	return (count == 9) ? TRUE : FALSE;
}

/*------------------------------------------------------------------------
Procedure:     on_drawingarea1_expose_event ID:1
Purpose:       Draws the grid and marks every time an Expose event occurs.
Input:
Output:
Errors:
------------------------------------------------------------------------*/
gboolean on_drawingarea1_expose_event (GtkWidget *widget, GdkEventExpose *event, gpointer user_data)
{
	int i,x,y,stepx,stepy;

	/* draw grid */
	stepx = (widget->allocation.width - 1) / 3;
	stepy = (widget->allocation.height - 1)/ 3;

	for (i=0; i < 4; i++) {
		gdk_draw_line (widget->window,
			widget->style->black_gc,
			0,
			i * stepy,
			stepx * 3,
			i* stepy);
		gdk_draw_line (widget->window,
			widget->style->black_gc,
			i * stepx,
			0,
			i * stepx,
			stepy * 3);
	}

	/* draw marks */
	for (x = 0; x < 4; x++)
		for (y = 0; y < 4; y++) {
			switch (sheet[x][y]) {
			case NONE:
				draw_blank(widget, x * stepx, y * stepy, stepx, stepy);
				break;
			case CROSS:
				draw_cross(widget, x * stepx, y * stepy, stepx, stepy);
				break;
			case CIRCLE:
				draw_circle(widget, x * stepx, y * stepy, stepx, stepy);
				break;
			}
		}

	return FALSE;
}

gboolean on_drawingarea1_button_release_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data)
{
	int stepx, stepy, x, y, count;

	if (win)
		return FALSE;

	/* translate mouse position to grid coordinates */
	stepx = (widget->allocation.width - 1) / 3;
	stepy = (widget->allocation.height - 1)/ 3;

	x = event->x / stepx;
	y = event->y / stepy;

	if ((x < 0) || (x > 2) || (y < 0) || (y > 2))
		return FALSE;

	/* check if field is free and put mark there */
	if (sheet[x][y] == NONE)
		sheet[x][y] = CROSS;
	else
	    return FALSE;

	gtk_widget_queue_draw(widget);

	/* check if a win */
	if (check_win(CROSS)) {
		show_message("You won!");
		win = TRUE;
		return FALSE;
	}

	/* check if a draw */
	if (check_draw()) {
		show_message("Draw!");
		win = TRUE;
		return FALSE;
	}

	/* computer's turn */
	for (count = 0; count < 100; count++) {
		x = (int) (3.0 * rand() / (RAND_MAX + 1.0));
		y = (int) (3.0 * rand() / (RAND_MAX + 1.0));
		if (sheet[x][y] == NONE) {
			sheet[x][y] = CIRCLE;
			break;
		}
	}
	if (count > 99) {
		show_message("I give up");
		win = 1;
		return FALSE;
	}


	gtk_widget_queue_draw(widget);

	/* check if a win */
	if (check_win(CIRCLE)) {
		show_message("I won!");
		win = TRUE;
		return FALSE;
	}

	/* check if a draw */
	if (check_draw()) {
		show_message("Draw!");
		win = TRUE;
		return FALSE;
	}

	return FALSE;
}


void on_button1_pressed (GtkButton *button, gpointer user_data)
{
	reset_game();
}


void on_button2_pressed (GtkButton *button, gpointer user_data)
{
	gtk_main_quit();
}

void on_root_window_destroy (GtkWindow *window, gpointer user_data)
{
	gtk_main_quit();
}


/*------------------------------------------------------------------------
Procedure:     create_root_window
Purpose:       Creates the widget hierarchy, connects signal handlers
Input:
Output:        Root window
Errors:
------------------------------------------------------------------------*/
GtkWidget* create_root_window ()
{
	GtkWidget *root_window;
	GtkWidget *vbox1;
	GtkWidget *button1;
	GtkWidget *button2;

	root_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_object_set_data (GTK_OBJECT (root_window), "root_window", root_window);
	gtk_window_set_title (GTK_WINDOW (root_window), "Tic Tac Toe");
	gtk_window_set_policy (GTK_WINDOW (root_window), TRUE, TRUE, FALSE);

	vbox1 = gtk_vbox_new (FALSE, 0);
	gtk_widget_ref (vbox1);
	gtk_object_set_data_full (GTK_OBJECT (root_window), "vbox1", vbox1,
		(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (vbox1);
	gtk_container_add (GTK_CONTAINER (root_window), vbox1);

	drawingarea1 = gtk_drawing_area_new ();
	gtk_widget_ref (drawingarea1);
	gtk_object_set_data_full (GTK_OBJECT (root_window), "drawingarea1", drawingarea1,
		(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (drawingarea1);
	gtk_box_pack_start (GTK_BOX (vbox1), drawingarea1, TRUE, TRUE, 4);
	gtk_widget_set_events (drawingarea1, GDK_EXPOSURE_MASK | GDK_BUTTON_RELEASE_MASK);
	gtk_widget_set_usize (drawingarea1, 120, 120);

	button1 = gtk_button_new_with_label ("Start Over");
	gtk_widget_ref (button1);
	gtk_object_set_data_full (GTK_OBJECT (root_window), "button1", button1,
		(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (button1);
	gtk_box_pack_start (GTK_BOX (vbox1), button1, FALSE, FALSE, 3);

	button2 = gtk_button_new_with_label ("Quit");
	gtk_widget_ref (button2);
	gtk_object_set_data_full (GTK_OBJECT (root_window), "button2", button2,
		(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (button2);
	gtk_box_pack_start (GTK_BOX (vbox1), button2, FALSE, FALSE, 3);

	statusbar1 = gtk_statusbar_new ();
	gtk_widget_ref (statusbar1);
	gtk_object_set_data_full (GTK_OBJECT (root_window), "statusbar1", statusbar1,
		(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (statusbar1);
	gtk_box_pack_start (GTK_BOX (vbox1), statusbar1, FALSE, FALSE, 3);

	gtk_signal_connect (GTK_OBJECT (drawingarea1), "button_release_event",
		GTK_SIGNAL_FUNC (on_drawingarea1_button_release_event),
		NULL);
	gtk_signal_connect (GTK_OBJECT (drawingarea1), "expose_event",
		GTK_SIGNAL_FUNC (on_drawingarea1_expose_event),
		NULL);
	gtk_signal_connect (GTK_OBJECT (button1), "pressed",
		GTK_SIGNAL_FUNC (on_button1_pressed),
		NULL);
	gtk_signal_connect (GTK_OBJECT (button2), "pressed",
		GTK_SIGNAL_FUNC (on_button2_pressed),
		NULL);
	gtk_signal_connect (GTK_OBJECT (root_window), "destroy",
		GTK_SIGNAL_FUNC (on_root_window_destroy),
		NULL);
		
	return root_window;
}

/*------------------------------------------------------------------------
Procedure:     main ID
Purpose:       Inits GTK, creates and displays the widget hierarchy for
the application and calls GTK event loop.
Input:         All command-line arguments are passed to GTK
Output:
Errors:
------------------------------------------------------------------------*/
int main (int argc, char *argv[])
{
	GtkWidget *root_window;

	/* initialize random number generator */
	srand(time(NULL));

	gtk_init (&argc, &argv);

	root_window = create_root_window ();
	gtk_widget_show (root_window);

	reset_game();

	gtk_main ();
	return 0;
}

